RollMaster = LibStub("AceAddon-3.0"):NewAddon("RollMaster", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("RollMaster", true)
local df = LibStub("LibDeformat-3.0")
local rm = RollMaster
local defaults = {
  profile = {
    timeout = 30,
    keptrolls = 5,
    groupOutput = true,
  }
}
local options = {
    name = L["RollMaster"],
    handler = RollMaster,
    type = "group",
  	get = function(info) return rm.db[info[#info]] end,
  	set = function(info, value) rm.db[info[#info]] = value end,
    args = {
      timeout = {
        name = L["Timeout"],
        desc = L["The time from a loot roll starting to it timing out"],
        type = "range",
        min = 10,
        max = 60,
        step = 1,
      },
      keptrolls = {
        name = L["Kept Rolls"],
        desc = L["The number of historical rolls to keep"],
        type = "range",
        min = 0,
        max = 20,
        step = 1,
      },
      groupOutput = {
        name = L["Group Output"],
        desc = L["When grouped, tell the group about the roll and result"],
        type = "toggle",
      },
      list = {
        name = L["List rolls"],
        desc = L["List stored rolls"],
        type = "execute",
        func = "ListRolls",
        hidden = true,
      },
      printRoll = {
        name = L["Display roll"],
        desc = L["Print a specific historic roll"],
        type = "execute",
        func = "DisplayRoll",
        hidden = true,
      },
    },
}


function RollMaster:OnInitialize()
  -- Code that you want to run when the addon is first loaded goes here.
  self.db_root = LibStub("AceDB-3.0"):New("RollMasterDB",defaults)
  self.db=self.db_root.profile
  if not self.db.groupOutput then self.db.groupOutput=true end
	local acreg = LibStub("AceConfigRegistry-3.0")
	acreg:RegisterOptionsTable("RollMaster", options)
	acreg:RegisterOptionsTable("RollMaster Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db_root))
	local acdia = LibStub("AceConfigDialog-3.0")
	acdia:AddToBlizOptions("RollMaster", L["RollMaster"])
	acdia:AddToBlizOptions("RollMaster Profiles", L["Profiles"], "RollMaster")
  self:RegisterChatCommand("rollmaster", "ChatCommand")
  self:RegisterChatCommand("rm", "ChatCommand")
end

function RollMaster:OnEnable()
  self:RegisterEvent("CHAT_MSG_SYSTEM")
end

function RollMaster:OnDisable()
  -- Do nothing
end

function RollMaster:ChatCommand(input)
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory("RollMaster")
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(RollMaster, "rm", "RollMaster", input)
  end
end

function RollMaster:CHAT_MSG_SYSTEM(event, msg)
  local player,rolled,low,high = df:Deformat(msg, RANDOM_ROLL_RESULT)
  if player then
    self:ProcessRoll(player,rolled,low,high)
  end
end

local function getRollCount()
  local rmcnt=GetNumRaidMembers()
  local ptcnt=GetNumPartyMembers()
  if rmcnt>0 then 
    return rmcnt+1
  else
    return ptcnt+1
  end
end

function RollMaster:output(msg,selfOnly)
  local toGroup=self.db.groupOutput and not selfOnly
  local dest=nil
  if toGroup and GetNumRaidMembers()>0 then
    dest="RAID"
  elseif toGroup and GetNumPartyMembers()>1 then
    dest="PARTY"
  else
    dest=nil
  end
  if dest then
    SendChatMessage(msg,dest)
  else
    self:Print(msg)
  end
end

function RollMaster:ProcessRoll(pl,rl,lo,hi)
  local cr=self.currentRoll or {}
  table.insert(cr,{pl,rl,lo,hi})
  self.currentRoll=cr
  if #cr==1 then
    self.rollCount=getRollCount()
    self.timestamp=date()
    self.rollTimer=self:ScheduleTimer("RollComplete",self.db.timeout,L["Time Up"])
    self:output(L["roll start"](self.rollCount,self.db.timeout))
  end
  if #cr==self.rollCount then
    self:RollComplete(L["All rolled"])
  end
end

local function printroll(roll,rollIdx,selfOnly)
  local header=L["roll header"](roll.timestamp,roll.reason)
  if rollIdx then
    header=format("%d. %s",rollIdx,header)
  end
  rm:output(header,selfOnly)
  for idx,roll in ipairs(roll) do
    rm:output(L["ranked roll"](idx,roll[1],roll[2],roll[3],roll[4]),selfOnly)
  end
end

function RollMaster:RollComplete(reason)
  table.sort(self.currentRoll, function(a,b) return a[2]>b[2] end)
  self.currentRoll.reason=reason
  self.currentRoll.timestamp=self.timestamp
  printroll(self.currentRoll,nil,false)
  self.db.oldrolls=self.db.oldrolls or {}
  table.insert(self.db.oldrolls,self.currentRoll)
  self.currentRoll=nil
  self:CancelTimer(self.rollTimer,true)
  self.rollTimer=nil
  if #self.db.oldrolls>self.db.keptrolls then
    table.remove(self.db.oldrolls,1)
  end
end

function RollMaster:DisplayRoll(index)
  if not self.db.oldrolls or #self.db.oldrolls<index then
    self:output(L["no such roll index"](index),true)
  else
    self:printroll(self.db.oldrolls[index],nil,false)
  end
end

function RollMaster:ListRolls()
  if not self.db.oldrolls then
    self:output(L["No rolls stored"],true)
  else
    for idx,roll in ipairs(self.db.oldrolls) do
      printroll(roll,idx,true)
    end
  end
end  
    
